#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# QuickRecon v0.2.3
# Copyright (C) 2011 Filip Szymański <f.szymanski@hotmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# For help run 'python quickrecon.py'
#
# -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#
# Please note that this tool was created and published for educational purposes only.
#
# -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

import getopt
import httplib
import os
import random
import re
import socket
import StringIO
import sys
import time
import urllib2

buffer = StringIO.StringIO()

usage = u'''
 ****************************
 * QuickRecon v0.2.3        *
 * Coded by Filip Szymański *
 * f.szymanski@hotmail.com  *
 ****************************

 Usage: quickrecon [options]
 
                      -m, --mode= {s, z, e, r}   Working mode.

                                        s        Bruteforce subdomains.
                                        z        Zone transfer.
                                        e        Gather email addresses from Google and Bing.
                                        r        Find human relationships using XHTML Friends Network.

                      -d, --domain=              Domain name.
                      -u, --url=                 Blogs url.
                      -l, --limit=               Google and Bing search limit (default = 50).
                      -i, --input=               Input file.
                      -o, --output=              Output file.

 Examples:            quickrecon.py -m s -d example.com -o out.log
                      quickrecon.py -m s -d example.com -i your_subdomains.txt
                      quickrecon.py -m z -d example.com
                      quickrecon.py -m e -d example.com -o emails.log
                      quickrecon.py -m e -d example.com -l 200
                      quickrecon.py -m r -u http://blog.example.com/
			'''

def find_subdomains(ifile):
	test_wildcard()

	found = 0

	try:
		file = open(ifile, 'r')
	except IOError:
		print 'No such file or directory: \'%s\'.' % ifile
		sys.exit(2)

	print ' / SUBDOMAIN: /\n'
	buffer.write(' / SUBDOMAIN: /\n\n')

	for line in file.readlines():
		sub = line.rstrip('\n')
		subdomain = sub + '.' + domain
		try:
			ip = socket.gethostbyname(subdomain)
			found = found + 1
		#	reverse = socket.gethostbyaddr(ip)
			print subdomain #, ip, reverse[0]
			buffer.write(subdomain + '\n')
		except:
			pass

	print '\n  Found %s subdomain(s).' % found
	buffer.write('\n  Found ' + str(found) + ' subdomain(s).\n')


def test_wildcard():
	container = []
	scode = []
	charset = 'abcdefghijklmnopqrstuvwxyz0123456789'

	for count in range(3):
		rcharset = random.sample(charset, random.randint(6, 12))
		rsub = ''.join(rcharset)
		rsubdomain = rsub + '.' + domain
		container.append(rsubdomain)

	for tsubdomain in container:
		try:
			conn = httplib.HTTPConnection(tsubdomain)
			conn.putrequest('GET', '/')
			conn.putheader('User-Agent', 'Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1)')
			conn.endheaders()
			response = conn.getresponse()
		#	print response.msg
			scode.append(response.status)
		except:
			pass

	if len(scode) > 0:
		print 'The target domain has a DNS wildcard configuration.'
		sys.exit(1)

def zone_transfer():
	try:
		import dns.resolver
		import dns.query
		import dns.zone
	except ImportError:
		print 'Please install \'dnspython\' from http://www.dnspython.org/.'
		sys.exit(2)

	ns = []
	data = ''

	print ' / ZONE TRANSFER: /\n'
	buffer.write(' / ZONE TRANSFER: /\n\n')

	try:
		nsrecords = dns.resolver.query(domain, 'NS')
	except dns.resolver.NXDOMAIN:
		print '  Zone transfer is not possible.'
		buffer.write('  Zone transfer is not possible.\n')
		return

	for server in nsrecords:
		ns.append(str(server))

	for target in ns:
		try:
			zt = dns.zone.from_xfr(dns.query.xfr(target.rstrip('.'), domain))
			names = zt.nodes.keys()
			names.sort()
			for name in names:
				data += zt[name].to_text(name)
				print re.sub(' ', '   ', zt[name].to_text(name))
				buffer.write(re.sub(' ', '   ', zt[name].to_text(name)) + '\n')
		except:
			pass

	if data == '':
		print '  Zone transfer is not possible.'
		buffer.write('  Zone transfer is not possible.\n')

def generate_urls(limit):
	global urls

	urls = []

	for ngoogle in range(0, limit, 10):
		urls.append('http://www.google.com/search?hl=en&ie=UTF-8&num=10&q=%40' + domain + '&start=' + str(ngoogle))
		urls.append('http://groups.google.com/groups?hl=en&ie=UTF-8&num=10&q=%40' + domain + '&start=' + str(ngoogle))

	for nbing in range(1, limit + 1, 10):
		urls.append('http://www.bing.com/search?q=%40' + domain + '&go=&count=10&first=' + str(nbing))

def find_emails(urls):
	patterns = [re.compile('[\w.]+@' + string + domain) for string in ['<b>', '<strong>']]
	results = {}

	print ' / EMAIL: /\n'
	buffer.write(' / EMAIL: /\n\n')

	for url in urls:
		try:
			request = urllib2.Request(url)
			request.add_header('User-agent', 'Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1)')
			response = urllib2.urlopen(request)
			data = response.read()
			for pattern in patterns:
				rawemails = re.findall(pattern, data)
				for email in rawemails:
					uemail = re.sub('<b>|<strong>', '', email)
					results[uemail] = 1
		except httplib.IncompleteRead:
			print 'Error while reading data. Try again.'
			sys.exit(2)

	for key in results.keys():
		print key
		buffer.write(key + '\n')

	print '\n  Found %s email(s).' % str(len(results))
	buffer.write('\n  Found ' + str(len(results)) + ' email(s).\n')

def find_human_relationships(url):
	import HTMLParser
	try:
		import BeautifulSoup
	except ImportError:
		print 'Please install \'BeautifulSoup\' from http://www.crummy.com/software/BeautifulSoup/.'
		sys.exit(2)

	results = {}
	xfn_values = [
					'friend', 'acquaintance', 'contact', 'met', 'co-worker', 'colleague',
					'co-resident', 'neighbor', 'child', 'parent', 'sibling', 'spouse',
					'kin', 'muse', 'crush', 'date', 'sweetheart', 'me'
					]

	print ' / RESULT: /\n'
	buffer.write(' / RESULT: /\n\n')

	try:
		response = urllib2.urlopen(url)
	except:
		print 'Connection problem. Try again.'
		sys.exit(2)

	try:
		rawdata = BeautifulSoup.BeautifulSoup(response)
	except HTMLParser.HTMLParseError:
		print 'Error while reading data. Try again.'
		sys.exit(2)

	data = rawdata.findAll('a')

	for item in data:
		for value in xfn_values:
			if item.has_key('rel') and re.search(value, item['rel']) != None:
				results[item.string] = item['href'], item['rel']

	if len(results) == 0:
		print '  Nothing found.'
		buffer.write('  Nothing found.\n')
	else:
		for key, val in results.items():
			print 'NAME:          ' + key
			buffer.write('NAME:          ' + key + '\n')
			print 'URL:           ' + val[0]
			buffer.write('URL:           ' + val[0] + '\n')
			print 'RELATIONSHIP:  ' + re.sub(' ', ', ', val[1]) + '\n'
			buffer.write('RELATIONSHIP:  ' + re.sub(' ', ', ', val[1]) + '\n\n')

def save_output(ofile):
	try:
		log = open(ofile, 'w')
	except IOError:
		print 'No such directory: \'%s\'.' % os.path.dirname(ofile)
		sys.exit(2)

	log.write('# Logfile: ' + time.strftime('%a, %d %b %Y, %H:%M:%S\n\n'))
	log.write(buffer.getvalue())
	log.close()


def main(argv):
	global domain

	limit = 50
	ifile = 'subdomain.plugin'
	ofile = None

	if len(sys.argv) < 2:
		print usage
		sys.exit(1)

	try:
		options, arguments = getopt.getopt(argv, 'm:d:u:l:i:o:', ['mode=', 'domain=', 'url=', 'limit=', 'input=', 'output='])
	except getopt.GetoptError:
		print 'Option not recognized. Try again.'
	#	print usage
		sys.exit(2)
	for option, argument in options:
		if option in ('-m', '--mode'):
			mode = str(argument)
		elif option in ('-d', '--domain'):
			domain = str(argument)
		elif option in ('-u', '--url'):
			url = str(argument)
		elif option in ('-l', '--limit'):
			limit = int(argument)
		elif option in ('-i', '--input'):
			ifile = str(argument)
		elif option in ('-o', '--output'):
			ofile = str(argument)

	try:
		if mode == 's':
			find_subdomains(ifile)
		elif mode == 'z':
			zone_transfer()
		elif mode == 'e':
			generate_urls(limit)
			find_emails(urls)
		elif mode == 'r':
			find_human_relationships(url)
		else:
			print 'Mode not recognized. Try again.'
		#	print usage
			sys.exit(1)
	except UnboundLocalError:
		print 'Please select working mode.'
	#	print usage
		sys.exit(2)

	if ofile != None:
		save_output(ofile)

if __name__ == '__main__':
	try:
		main(sys.argv[1:])
	except KeyboardInterrupt:
		print '\nInterrupted by user...'
		sys.exit(2)
	finally:
		buffer.close()
